import { d as dirname, b as basename, k as resolve, p as mergeSlashes, q as join, s as slash, m as toFilePath } from './index-9fdde2e8.js';
import { c as createBirpc } from './index-e909c175.js';
import { c as distDir } from './constants-8b3a9b31.js';
import { builtinModules, createRequire } from 'module';
import { pathToFileURL, fileURLToPath } from 'url';
import vm from 'vm';
import { s as shouldExternalize } from './externalize-2f63779d.js';
import { existsSync, readdirSync } from 'fs';
import { a as spyOn, s as spies } from './jest-mock-038a01b3.js';
import { r as rpc } from './rpc-8c7cc374.js';
import 'tty';
import 'local-pkg';
import 'path';
import 'assert';
import 'util';
import 'chai';
import 'tinyspy';

var __defProp = Object.defineProperty;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
function resolveMockPath(mockPath, root, nmName) {
  if (nmName) {
    const mockDirname = dirname(nmName);
    const baseFilename = basename(nmName);
    const mockFolder = resolve(root, "__mocks__", mockDirname);
    if (!existsSync(mockFolder))
      return null;
    const files = readdirSync(mockFolder);
    for (const file of files) {
      const [basename2] = file.split(".");
      if (basename2 === baseFilename)
        return resolve(mockFolder, file).replace(root, "");
    }
    return null;
  }
  const dir = dirname(mockPath);
  const baseId = basename(mockPath);
  const fullPath = resolve(dir, "__mocks__", baseId);
  return existsSync(fullPath) ? fullPath.replace(root, "") : null;
}
function getObjectType(value) {
  return Object.prototype.toString.apply(value).slice(8, -1);
}
function mockPrototype(proto) {
  if (!proto)
    return null;
  const newProto = {};
  const protoDescr = Object.getOwnPropertyDescriptors(proto);
  for (const d in protoDescr) {
    Object.defineProperty(newProto, d, protoDescr[d]);
    if (typeof protoDescr[d].value === "function")
      spyOn(newProto, d).mockImplementation(() => {
      });
  }
  return newProto;
}
function mockObject(obj) {
  const type = getObjectType(obj);
  if (Array.isArray(obj))
    return [];
  else if (type !== "Object" && type !== "Module")
    return obj;
  const newObj = __spreadValues({}, obj);
  const proto = mockPrototype(Object.getPrototypeOf(obj));
  Object.setPrototypeOf(newObj, proto);
  for (const k in obj) {
    newObj[k] = mockObject(obj[k]);
    const type2 = getObjectType(obj[k]);
    if (type2.includes("Function") && !obj[k].__isSpy) {
      spyOn(newObj, k).mockImplementation(() => {
      });
      Object.defineProperty(newObj[k], "length", { value: 0 });
    }
  }
  return newObj;
}
function createMocker(root, mockMap) {
  function getSuiteFilepath() {
    var _a;
    return (_a = process.__vitest_worker__) == null ? void 0 : _a.filepath;
  }
  function getActualPath(path, nmName) {
    return nmName ? mergeSlashes(`/@fs/${path}`) : path.replace(root, "");
  }
  function unmockPath(path, nmName) {
    const suitefile = getSuiteFilepath();
    if (suitefile) {
      const fsPath = getActualPath(path, nmName);
      mockMap[suitefile] ?? (mockMap[suitefile] = {});
      delete mockMap[suitefile][fsPath];
    }
  }
  function mockPath(path, nmName, factory) {
    const suitefile = getSuiteFilepath();
    if (suitefile) {
      const fsPath = getActualPath(path, nmName);
      mockMap[suitefile] ?? (mockMap[suitefile] = {});
      mockMap[suitefile][fsPath] = factory || resolveMockPath(path, root, nmName);
    }
  }
  function clearMocks({ clearMocks: clearMocks2, mockReset, restoreMocks }) {
    if (!clearMocks2 && !mockReset && !restoreMocks)
      return;
    spies.forEach((s) => {
      if (restoreMocks)
        s.mockRestore();
      else if (mockReset)
        s.mockReset();
      else if (clearMocks2)
        s.mockClear();
    });
  }
  function resolveDependency(dep) {
    if (dep.startsWith("/node_modules/"))
      return mergeSlashes(`/@fs/${join(root, dep)}`);
    return dep;
  }
  return {
    mockPath,
    unmockPath,
    clearMocks,
    getActualPath,
    mockObject,
    getSuiteFilepath,
    resolveMockPath,
    resolveDependency
  };
}

const stubRequests = {
  "/@vite/client": {
    injectQuery: (id) => id,
    createHotContext() {
      return {
        accept: () => {
        },
        prune: () => {
        }
      };
    },
    updateStyle() {
    }
  }
};
function hasNestedDefault(target) {
  return "__esModule" in target && target.__esModule && "default" in target.default;
}
function proxyMethod(name, isNested) {
  return function(target, key, ...args) {
    const result = Reflect[name](target, key, ...args);
    if (isNested && key === "default" || !result)
      return Reflect[name](target.default, key, ...args);
    return result;
  };
}
async function interpretedImport(path, interpretDefault) {
  const mod = await import(path);
  if (interpretDefault && "default" in mod) {
    const isNested = hasNestedDefault(mod);
    return new Proxy(mod, {
      get: proxyMethod("get", isNested),
      set: proxyMethod("set", isNested),
      has: proxyMethod("has", isNested),
      deleteProperty: proxyMethod("deleteProperty", isNested)
    });
  }
  return mod;
}
async function executeInViteNode(options) {
  const { moduleCache, root, files, fetch, mockMap } = options;
  const externalCache = new Map();
  builtinModules.forEach((m) => externalCache.set(m, m));
  const {
    getActualPath,
    getSuiteFilepath,
    mockObject,
    mockPath,
    clearMocks,
    unmockPath,
    resolveMockPath,
    resolveDependency
  } = createMocker(root, mockMap);
  const result = [];
  for (const file of files)
    result.push(await cachedRequest(`/@fs/${slash(resolve(file))}`, []));
  return result;
  async function callFunctionMock(dep, mock) {
    var _a;
    const name = `${dep}__mock`;
    const cached = (_a = moduleCache.get(name)) == null ? void 0 : _a.exports;
    if (cached)
      return cached;
    const exports = await mock();
    setCache(name, { exports });
    return exports;
  }
  async function directRequest(id, fsPath, callstack) {
    callstack = [...callstack, id];
    const suite = getSuiteFilepath();
    const request = async (dep, canMock = true) => {
      var _a;
      if (canMock) {
        const mocks2 = mockMap[suite || ""] || {};
        const mock = mocks2[resolveDependency(dep)];
        if (typeof mock === "function")
          return callFunctionMock(dep, mock);
        if (typeof mock === "string")
          dep = mock;
      }
      if (callstack.includes(dep)) {
        const cacheKey = toFilePath(dep, root);
        if (!((_a = moduleCache.get(cacheKey)) == null ? void 0 : _a.exports))
          throw new Error(`Circular dependency detected
Stack:
${[...callstack, dep].reverse().map((p) => `- ${p}`).join("\n")}`);
        return moduleCache.get(cacheKey).exports;
      }
      return cachedRequest(dep, callstack);
    };
    if (id in stubRequests)
      return stubRequests[id];
    const transformed = await fetch(id);
    if (transformed == null)
      throw new Error(`failed to load ${id}`);
    const url = pathToFileURL(fsPath).href;
    const exports = {};
    setCache(fsPath, { code: transformed, exports });
    const __filename = fileURLToPath(url);
    const moduleProxy = {
      set exports(value) {
        exportAll(exports, value);
        exports.default = value;
      },
      get exports() {
        return exports.default;
      }
    };
    const importActual = (path, nmName) => {
      return request(getActualPath(path, nmName), false);
    };
    const importMock = async (path, nmName) => {
      if (!suite)
        throw new Error("You can import mock only inside of a running test");
      const mock = (mockMap[suite] || {})[path] || resolveMockPath(path, root, nmName);
      if (mock === null) {
        const fsPath2 = getActualPath(path, nmName);
        const exports2 = mockObject(await request(fsPath2, false));
        setCache(fsPath2, { exports: exports2 });
        return exports2;
      }
      if (typeof mock === "function")
        return callFunctionMock(path, mock);
      return request(mock, true);
    };
    const context = {
      __vite_ssr_import__: request,
      __vite_ssr_dynamic_import__: request,
      __vite_ssr_exports__: exports,
      __vite_ssr_exportAll__: (obj) => exportAll(exports, obj),
      __vite_ssr_import_meta__: { url },
      __vitest__mock__: mockPath,
      __vitest__unmock__: unmockPath,
      __vitest__importActual__: importActual,
      __vitest__importMock__: importMock,
      __vitest__clearMocks__: clearMocks,
      require: createRequire(url),
      exports,
      module: moduleProxy,
      __filename,
      __dirname: dirname(__filename)
    };
    const fn = vm.runInThisContext(`async (${Object.keys(context).join(",")})=>{{${transformed}
}}`, {
      filename: fsPath,
      lineOffset: 0
    });
    await fn(...Object.values(context));
    const mocks = suite ? mockMap[suite] : null;
    if (mocks) {
      if (mocks[id] === null)
        exportAll(exports, mockObject(exports));
    }
    return exports;
  }
  function setCache(id, mod) {
    if (!moduleCache.has(id))
      moduleCache.set(id, mod);
    else
      Object.assign(moduleCache.get(id), mod);
  }
  async function cachedRequest(rawId, callstack) {
    var _a, _b;
    const id = normalizeId(rawId);
    if (externalCache.get(id))
      return interpretedImport(patchWindowsImportPath(id), options.interpretDefault);
    const fsPath = toFilePath(id, root);
    const importPath = patchWindowsImportPath(fsPath);
    if (!externalCache.has(importPath))
      externalCache.set(importPath, await shouldExternalize(importPath, options));
    const externalId = externalCache.get(importPath);
    if (externalId)
      return interpretedImport(externalId, options.interpretDefault);
    if ((_a = moduleCache.get(fsPath)) == null ? void 0 : _a.promise)
      return (_b = moduleCache.get(fsPath)) == null ? void 0 : _b.promise;
    const promise = directRequest(id, fsPath, callstack);
    setCache(fsPath, { promise });
    return await promise;
  }
  function exportAll(exports, sourceModule) {
    for (const key in sourceModule) {
      if (key !== "default") {
        try {
          Object.defineProperty(exports, key, {
            enumerable: true,
            configurable: true,
            get() {
              return sourceModule[key];
            }
          });
        } catch (_err) {
        }
      }
    }
  }
}
function normalizeId(id) {
  return id.replace(/^\/@id\/__x00__/, "\0").replace(/^\/@id\//, "").replace(/^__vite-browser-external:/, "").replace(/^node:/, "").replace(/[?&]v=\w+/, "?").replace(/\?$/, "");
}
function patchWindowsImportPath(path) {
  if (path.match(/^\w:\\/))
    return `file:///${slash(path)}`;
  else if (path.match(/^\w:\//))
    return `file:///${path}`;
  else
    return path;
}

let _viteNode;
const moduleCache = new Map();
const mockMap = {};
async function startViteNode(ctx) {
  if (_viteNode)
    return _viteNode;
  const processExit = process.exit;
  process.on("beforeExit", (code) => {
    rpc().onWorkerExit(code);
  });
  process.exit = (code = process.exitCode || 0) => {
    rpc().onWorkerExit(code);
    return processExit(code);
  };
  const { config } = ctx;
  const { run: run2, collect: collect2 } = (await executeInViteNode({
    files: [
      resolve(distDir, "entry.js")
    ],
    fetch(id) {
      return rpc().fetch(id);
    },
    moduleCache,
    mockMap,
    root: config.root,
    depsInline: config.depsInline,
    depsExternal: config.depsExternal,
    fallbackCJS: config.fallbackCJS,
    interpretDefault: config.interpretDefault
  }))[0];
  _viteNode = { run: run2, collect: collect2 };
  return _viteNode;
}
function init(ctx) {
  if (process.__vitest_worker__ && ctx.config.threads)
    throw new Error(`worker for ${ctx.files.join(",")} already initialized by ${process.__vitest_worker__.ctx.files.join(",")}. This is probably an internal bug of Vitest.`);
  process.stdout.write("\0");
  const { config, port } = ctx;
  process.__vitest_worker__ = {
    ctx,
    moduleCache,
    config,
    rpc: createBirpc({
      functions: {},
      eventNames: ["onUserLog", "onCollected", "onWorkerExit"],
      post(v) {
        port.postMessage(v);
      },
      on(fn) {
        port.addListener("message", fn);
      }
    })
  };
  if (ctx.invalidates)
    ctx.invalidates.forEach((i) => moduleCache.delete(i));
  ctx.files.forEach((i) => moduleCache.delete(i));
}
async function collect(ctx) {
  init(ctx);
  const { collect: collect2 } = await startViteNode(ctx);
  return collect2(ctx.files, ctx.config);
}
async function run(ctx) {
  init(ctx);
  const { run: run2 } = await startViteNode(ctx);
  return run2(ctx.files, ctx.config);
}

export { collect, run };
